#!/usr/bin/perl

# author: Kevin K.M. Chiu
# version: 2.0
# date: 07-01-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $AdminCGIs_dir = '/usr/admserv/cgi-bin/.cobalt';
my $Template_dir = '/usr/admserv/templates';
my %System_users = ('mail','1','httpd','1','root','1','gopher','1','squid','1','guest','1','anonymous','1','halt','1','ftp','1','nobody','1','operator','1','games','1','adm','1','daemon','1','bin','1','shutdown','1','lp','1','sync','1','news','1','uucp','1');

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::List;
use Cobalt::Vsite;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::User;
use Cobalt::Util;

# use strict;
use CGI;
use Locale::gettext;

# global variables
my $q=new CGI;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $page=$q->param( 'page' );
my $listName=$q->param( 'listname' );

my $group=$q->param( 'group' );

my $mailListListScriptPath="$AdminCGIs_dir/mailListList/mailListList.cgi";
my $mailListListScriptURL='/cgi-bin/.cobalt/mailListList/mailListList.cgi';
my $mailListModScriptURL='/cgi-bin/.cobalt/mailListMod/mailListMod.cgi';
my $forbiddenURL="/.cobalt/error/forbidden.html";

my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "group", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not a member of $group.\n";
}

if( $page eq 'maillistmod' ) { 
    return handleMailListModPage(); 
} else {
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    return genMailListModPage( \*STDOUT, $group );
}
1;


#
# event handler to modify mailing list page
#
sub handleMailListModPage {
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>';

    # read form input
    my $group=$q->param( 'group' );
    my $oldListName=$q->param( 'oldlistname' );
    my $recipientsArea=$q->param( 'recipients' );
    my $postingsArea=$q->param( 'postings' );
    my @userRecipients=$q->param( 'userrecipients' );
    my $password=$q->param( 'password' );
    my $open=$q->param( 'open' );
    my $moderator=$q->param( 'moderator' );
    my $mod=$q->param( 'mod' );
    my $spam=$q->param( 'spam' );

    # drop the first element (blank)
    if ($userRecipients[0] eq '') {
	shift @userRecipients;
    }

    # get individual recipients
    my @recipients=split /[\s,]/m, $recipientsArea;

    # error check
    my $recipientErr=0;
    my $allRecipientsOK=1;
    my @resultRecipients;
    my $recipient;
    foreach $recipient ( @recipients ) {
	next unless ($recipient);

	unless (netutil_test_emailaddr($recipient)) {
	    if ($allRecipientsOK) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "recipientsNotValid")."\"");
		$allRecipientsOK=0;
	    }
	    $info.=" $recipient";

	    $recipientErr=1;
	} else {
	    push @resultRecipients, $recipient;
	}
    }
    $info.='<BR>' unless ($allRecipientsOK);
    @recipients=@resultRecipients;

    # get individual postings
    my @postings=split /[\s,]/m, $postingsArea;

    my $postingErr=0;
    my $allPostingsOK=1;
    my @resultPostings;
    foreach (@postings) {
	next unless ($_);
	unless (netutil_test_emailaddr($_)) {
	    if ($allPostingsOK) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "postingsNotValid")."\"");
		$allPostingsOK=0;
	    }
	    $info.=" $_";
	    $postingErr=1;
	} else {
	    push @resultPostings, $_;
	}
    }
    $info.='<BR>' unless ($allPostingsOK);
    @postings=@resultPostings;

    # check if list name is ok
    my @usersInGroup = vsite_get_users($group);
    foreach ( @usersInGroup ) {
	if ($_ eq $listName) {
	    $err = 1;
	}
    }
    my @aliases = Cobalt::Meta::query("type"  => "users",
		                      "keys"  => ["aliases"],
		                      "where" => ["vsite", "=", "$group"]);
    foreach ( @aliases ) {
	if ($_ eq $listName) {
	    $err = 1;
	}
    }

    if ($err) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
    } elsif (! $listName) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noMailListName")."\"");
	$err=1;
    } elsif ( $System_users {$listName} ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    } else {
	# everything is fine

	# set recipients list
	my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	my $ret = list_modify ($group, $fqdn, $oldListName, $listName,
			       map("$_\@${fqdn}", @userRecipients), @recipients);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set postings list
	my $ret = list_set_post ($group, $fqdn, $listName, @postings);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set mailing list password
	$ret = list_config_password($group, $fqdn, $listName, $password);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
      
	# set the open subscription option
	if ($open) {
	    $ret=list_config_open($group, $fqdn, $listName);
	} else {
	    $ret=list_config_closed($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the moderator email address
	unless (!$moderator || netutil_test_emailaddr($moderator)) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "moderatorNotValid")."\"");
	    $info.=" $moderator<BR>";
	    $err=1;
	} else {
	    $ret=list_config_moderator($group, $fqdn, $listName, $moderator);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 );
		$err=1;
	    }
	}

	# set the moderated option
	if ($mod) {
	    $ret=list_config_moderate($group, $fqdn, $listName);
	} else {
	    $ret=list_config_nomoderate($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the restricted posting option
	if ($spam eq 0) {
	    $ret=list_config_norestrict($group, $fqdn, $listName);
	} elsif ($spam eq 1) {
	    $ret=list_config_restrict($group, $fqdn, $listName);
	} elsif ($spam eq 2) {
	    $ret=list_config_restrictPrivate($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
    }

    if ($err || $recipientErr || $postingErr) {
	# generate error message
	$ENV{ PARAM_GROUP }=$group;
	$ENV{ PARAM_INFO }=$info;
	require $mailListListScriptPath;
	delete $ENV{ PARAM_GROUP };
	delete $ENV{ PARAM_INFO };
    } else {
	# generate mailing lists list
	$ENV{ PARAM_GROUP }=$group;
	require $mailListListScriptPath;
	delete $ENV{ PARAM_GROUP };
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the infomation shown to user
#
sub genMailListModPage {
    my ( $handle, $group, $info )=@_;

    my $oldListName=$listName;
    my $mailListNameField="<INPUT TYPE=\"TEXT\" NAME=\"listname\" VALUE=\"$listName\" SIZE=\"15\" onChange=\"checkListName( this, mailListNameNotValid );\">";
    my $selectlist = MSG_get_ui("mailListSelectList");

    # get all list members
    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
    my @listMembers = list_get_mem($group, $fqdn, $listName);
    if ($listMembers[0] =~ /^[134]\S*\s/) {
	@listMembers = ();
    }

    # get all postings list
    my @listPostings = list_get_post($group, $fqdn, $listName);
    if ($listPostings[0] =~ /^[134]\S*\s/) {
	@listPostings = ();
    }
    my $postings=join "\n", sort @listPostings;

    my $mailListName = $oldListName . '@'. $fqdn;

    # get list password
    my $password = (Cobalt::List::list_config_getpassword($group, $fqdn, $listName));

    # get list subscription policy
    my $open = (Cobalt::List::list_config_getstate($group, $fqdn, $listName)) ? "CHECKED" : '';

    # get moderator email address
    my $moderator = (Cobalt::List::list_config_getmoderator($group, $fqdn, $listName));

    # get list moderation policy
    my $moderate = (Cobalt::List::list_config_getmoderate($group, $fqdn, $listName)) ? "CHECKED" : '';

    # get list restricted status
    my $spam = (Cobalt::List::list_config_getrestrict($group, $fqdn, $listName));
    my $spam0checked = ($spam eq 0) ? "CHECKED" : '';
    my $spam1checked = ($spam eq 1) ? "CHECKED" : '';
    my $spam2checked = ($spam eq 2) ? "CHECKED" : '';

    # do not show admin
    my @users=vsite_get_users($group);
    if ($users[0] =~ /^[134]\S*\s/) {
	@users = ();
    }
    my ($i,$selected,@usersF,$u,$userList);
    @users = (sort @users);
    if (scalar( @users)) {
	$userList = "<SELECT MULTIPLE NAME=\"userrecipients\" SIZE=6>\n";
	$userList .= "<OPTION VALUE=\"\">--- $selectlist ---</OPTION>\n";
	foreach (@users) {
	    $selected = '';
	    $u = $_ . '@' . $fqdn;
	    push @usersF, $u;
	    if (grep { /^$u$/ } @listMembers) {
		$selected = 'SELECTED';
	    }
	    $userList.="<OPTION VALUE=\"$_\" $selected>$_\@$fqdn</OPTION>\n";
	}
	$userList .="</SELECT>\n";
	$userList .=qq[ <CENTER><A HREF="javascript: setAllUsers();" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/.cobalt/images/select_all_but" BORDER="0"></A></CENTER>\n ];
    } else {
	my $noUsers=eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"");
	$userList = "<FONT COLOR=\"#FFFFFF\">&nbsp;</FONT><I>$noUsers</I>\n";
    }

    my $cancelURL = $mailListListScriptURL . "?group=$group";
    my $mailListOpenHelp=MSG_get_ui("mailListOpenHelp", 'majordomo@'.$fqdn);

    # find recipients
    my @recipientsList;
    my $lm;
    foreach $lm (@listMembers) {
	unless (grep { /^$lm$/ } @usersF) { push @recipientsList, $lm; }
    }
    my $recipients=join "\n", sort @recipientsList;

    # Read in the template
    open( TEMPLATE, "$Template_dir/mailListMod.tmpl" )
	|| die "Cannot read $Template_dir/mailListMod.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[MAILLISTLISTSCRIPTURL\]/$cancelURL/o;
	s/\[MAILLISTMODSCRIPTURL\]/$mailListModScriptURL/o;
	s/\[MAILLISTNAMEFIELD\]/$mailListNameField/;
	s/\[MAILLISTOPENHELP\]/$mailListOpenHelp/o;
	s/\[OLDMAILLISTNAME\]/$oldListName/;
	s/\[RECIPIENTS\]/$recipients/;
	s/\[POSTINGS\]/$postings/;
	s/\[USER_INFO\]/$info/o;
	s/\[GROUP\]/$group/;
	s/\[USERLIST\]/$userList/;
	s/\[PASSWORD\]/$password/;
	s/\[MAILLISTNAME\]/$mailListName/;
	s/\[OPENCHECKED\]/$open/;
	s/\[MODERATOR\]/$moderator/;
	s/\[MODCHECKED\]/$moderate/;
	s/\[SPAM0CHECKED\]/$spam0checked/;
	s/\[SPAM1CHECKED\]/$spam1checked/;
	s/\[SPAM2CHECKED\]/$spam2checked/;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $handle $_;
    }
    close TEMPLATE;
}
